@@ -18,6 +18,8 @@ Validation::Class::Exporter->apply_spec(
     routines => ['doc', 'document', 'validate_document', 'document_validates'],
 );
 
+my $i = 0; # IAMTHATIAM
+
 
 
 sub doc { goto &document } sub document {
@@ -49,14 +51,49 @@ sub doc { goto &document } sub document {
 
 sub document_validates { goto &validate_document } sub validate_document {
 
-    my ($self, $name, $data, $options) = @_;
+    my ($self, $ref, $data, $options) = @_;
+
+    my $name;
 
     my $proto     = $self->prototype;
 
-    my $fields    = { map {$_ => 1} ($proto->fields->keys) };
-
     my $documents = clone $proto->settings->get('documents');
 
+    my $_fmap     = {}; # ad-hoc fields
+
+    if ("HASH" eq ref $ref) {
+
+        $ref  = clone $ref;
+
+        $name = "DOC_" . time() . ++$i;
+
+        # build document on-the-fly from a hashref
+        foreach my $rules (values %{$ref}) {
+
+            next unless "HASH" eq ref $rules;
+
+            my  $id = uc "$rules";
+                $id =~ s/\W/_/g;
+                $id =~ s/_$//;
+
+            $proto->fields->add($id => $rules);
+            $rules = $id;
+            $_fmap->{$id} = 1;
+
+        }
+
+        $documents->{$name} = $ref;
+
+    }
+
+    else {
+
+        $name = $ref;
+
+    }
+
+    my $fields    = { map {$_ => 1} ($proto->fields->keys) };
+
     croak "Please supply a registered document name to validate against"
         unless $name
     ;
@@ -126,7 +163,17 @@ sub document_validates { goto &validate_document } sub validate_document {
 
                 if ($key =~ /^$regex$/) {
 
-                    $proto->clone_field($field => $point, {label => $label});
+                    my $config = {label => $label};
+
+                    $config->{mixin} = $proto->fields->get($field)->mixin
+                        if $proto->fields->get($field)->can('mixin')
+                    ;
+
+                    $proto->clone_field($field, $point => $config);
+
+                    $proto->apply_mixin($point => $config->{mixin})
+                        if $config->{mixin}
+                    ;
 
                     $_dmap->{$key}   = 1;
                     $_pmap->{$point} = $key;
@@ -165,6 +212,8 @@ sub document_validates { goto &validate_document } sub validate_document {
 
     $proto->validate($self);
 
+    $proto->clear_queue;
+
     my @errors = $proto->get_errors;
 
     for (sort @errors) {
@@ -194,6 +243,8 @@ sub document_validates { goto &validate_document } sub validate_document {
 
     }
 
+    $proto->fields->delete($_) for keys %{$_fmap}; # reap ad-hoc fields
+
     $proto->reset_fields;
 
     $proto->set_errors(@errors) if @errors; # report errors
@@ -216,7 +267,7 @@ Validation::Class::Document - Data Validation for Hierarchical Data
 
 =head1 VERSION
 
-version 0.000014
+version 0.000015
 
 =head1 SYNOPSIS
 
@@ -224,6 +275,157 @@ version 0.000014
 
     use Validation::Class::Document;
 
+    my $person = MyApp::Person->new;
+
+    # rules
+
+    my $schema = {
+        'id'                        => {mixin => [':num'], max_length => 4},
+        'name'                      => {mixin => [':str'], min_length => 2},
+        'title'                     => {mixin => [':str'], min_length => 5},
+        'company.name'              => {mixin => [':str'], min_length => 2},
+        'company.tags.@'            => {mixin => [':str'], min_length => 2},
+        'company.super*.name'       => {mixin => [':str'], min_length => 2},
+        'company.super*.rating.@.*' => {mixin => [':str'], },
+    };
+
+    # data
+
+    my $data = {
+        "id"      => "1234-ABC",
+        "name"    => "Anita Campbell-Green",
+        "title"   => "Designer",
+        "company" => {
+            "name"       => "House of de Vil",
+            "supervisor" => {
+                "name"   => "Cruella de Vil",
+                "rating" => [
+                    {   "support"  => -9,
+                        "guidance" => -9
+                    }
+                ]
+            },
+            "tags" => [
+                "evil",
+                "cruelty",
+                "dogs"
+            ]
+        },
+    };
+
+    unless ($person->validate_document($schema => $data)) {
+        warn $person->errors_to_string if $person->error_count;
+    }
+
+=head1 DESCRIPTION
+
+Validation::Class::Document inherits all functionality from L<Validation::Class>
+and implements additional abilities documented as follows.
+
+This module allows you to validate hierarchical structures using the
+L<Validation::Class> framework. This is an experimental yet highly promising
+approach toward the consistent processing of nested structures. The current
+interface is not expected to change. This module was inspired by
+L<MooseX::Validation::Doctypes>. Gone are the days of data submitted to an
+application in key/value form and especially in regards to the increasing demand
+for communication between applications, serializing and transmitting structured
+data over a network connection. Note! All matching data points will be validated
+forcefully (i.e. matching data points are required to have a value defined).
+
+=head1 OVERVIEW
+
+Validation::Class::Document exports the document (or dom) keyword which allows
+you to pre-define/configure your path matching rules for your data structure.
+The "path matching rules", which is actually a custom object notation, referred
+to as the document notation, can be thought of as a kind-of simplified regular
+expression which is executed against the flattened data structure. The following
+are a few general use-cases:
+
+    # given this JSON data structure
+    {
+        "id": "1234-A",
+        "name": {
+            "first_name" : "Bob",
+            "last_name"  : "Smith",
+         },
+        "title": "CIO",
+        "friends" : [],
+    }
+
+    # select id to validate against the string rules
+    document 'foobar'  =>
+        { 'id' => 'string' };
+
+    # select name -> first_name/last_name to validate against the string rules
+    document 'foobar'  =>
+        {'name.first_name' => 'string', 'name.last_name' => 'string'};
+
+    # or
+    document 'foobar'  =>
+        {'name.*_name' => 'string'};
+
+    # select each element in friends to validate against the string rules
+    document 'foobar'  =>
+        { 'friends.@'  => 'string' };
+
+    # or select an element of a hashref in each element in friends to validate
+    document 'foobar'  =>
+        { 'friends.@.name' => 'string' };
+
+The document declaration's keys should follow the aforementioned document
+notation schema and it's values should be strings which correspond to the names
+of fields (or other document declarations) that will be used to preform the
+data validation. It is possible to combine document declarations to validate
+hierarchical data that contains data structures matching one or more document
+patterns. The following is an example of what that might look like.
+
+    package MyApp::Person;
+
+    use Validation::Class::Document;
+
+    # data validation rule
+    field  'name' => {
+        mixin      => [':str'],
+        pattern    => qr/^[A-Za-z ]+$/,
+        max_length => 20,
+    };
+
+    # data validation map / document notation schema
+    document 'friend' => {
+        'name' => 'name'
+    };
+
+    # data validation map / document notation schema
+    document 'person' => {
+        'name' => 'name',
+        'friends.@' => 'friend'
+    };
+
+    package main;
+
+    my $data = {
+        "name"   => "Anita Campbell-Green",
+        "friends" => [
+            { "name" => "Horace" },
+            { "name" => "Skinner" },
+            { "name" => "Alonzo" },
+            { "name" => "Frederick" },
+        ],
+    };
+
+    my $person = MyApp::Person->new;
+
+    unless ($person->validate_document(person => $data)) {
+        warn $person->errors_to_string if $person->error_count;
+    }
+
+Alternatively, the following is a more verbose data validation class using
+traditional styling and configuration.
+
+    package MyApp::Person;
+
+    use Validation::Class::Document;
+
     field  'id' => {
         mixin      => [':str'],
         filters    => ['numeric'],
@@ -286,104 +488,6 @@ version 0.000014
         warn $person->errors_to_string if $person->error_count;
     }
 
-=head1 DESCRIPTION
-
-Validation::Class::Document inherits all functionality from L<Validation::Class>
-and implements additional abilities documented as follows.
-
-This module allows you to validate hierarchical structures using the
-L<Validation::Class> framework. This is an experimental yet highly promising
-approach toward the consistent processing of nested structures. The current
-interface is not expected to change. This module was inspired by
-L<MooseX::Validation::Doctypes>. Gone are the days of data submitted to an
-application in key/value form and especially in regards to the increasing demand
-for communication between applications, serializing and transmitting structured
-data over a network connection.
-
-=head1 OVERVIEW
-
-Validation::Class::Document exports the document (or dom) keyword which allows
-you to pre-define/configure your path matching rules for your data structure.
-The "path matching rules", which is actually a custom object notation, referred
-to as the document notation, can be thought of as a kind-of simplified regular
-expression which is executed against the flattened data structure. The following
-are a few general use-cases:
-
-    # given the data structure
-    {
-        "id": "1234-A",
-        "name": {
-            "first_name" : "Bob",
-            "last_name"  : "Smith",
-         },
-        "title": "CIO",
-        "friends" : [],
-    }
-
-    # select id to validate against the string rules
-    document 'foobar'  =>
-        { 'id' => 'string' };
-
-    # select name -> first_name/last_name to validate against the string rules
-    document 'foobar'  =>
-        {'name.first_name' => 'string', 'name.last_name' => 'string'};
-
-    # or
-    document 'foobar'  =>
-        {'name.*_name' => 'string'};
-
-    # select each element in friends to validate against the string rules
-    document 'foobar'  =>
-        { 'friends.@'  => 'string' };
-
-    # or select an element of a hashref in each element in friends to validate
-    document 'foobar'  =>
-        { 'friends.@.name' => 'string' };
-
-The document declaration's keys should follow the aforementioned document
-notation scheme and it's values should be strings which correspond to the names
-of fields (or other document declarations) that will be used to preform the
-data validation. It is possible to combine document declarations to validate
-hierarchical data that contains data structures matching one or more document
-patterns. The following is an example of what that might look like.
-
-    package MyApp::Person;
-
-    use Validation::Class::Document;
-
-    field  'name' => {
-        mixin      => [':str'],
-        pattern    => qr/^[A-Za-z ]+$/,
-        max_length => 20,
-    };
-
-    document 'friend' => {
-        'name' => 'name'
-    };
-
-    document 'person' => {
-        'name' => 'name',
-        'friends.@' => 'friend'
-    };
-
-    package main;
-
-    my $data = {
-        "name"   => "Anita Campbell-Green",
-        "friends" => [
-            { "name" => "Horace" },
-            { "name" => "Skinner" },
-            { "name" => "Alonzo" },
-            { "name" => "Frederick" },
-        ],
-    };
-
-    my $person = MyApp::Person->new;
-
-    unless ($person->validate_document(person => $data)) {
-        warn $person->errors_to_string if $person->error_count;
-    }
-
 =head1 METHODS
 
 =head2 validate_document
@@ -403,6 +507,10 @@ the validation process. The following is an example of this technique:
     # the prune option removes non-matching parameters (nodes)
     my $boolean = $self->validate_document(foobar => $data, { prune => 1 });
 
+Additionally, to support the validation of ad-hoc specifications, you may pass
+this method two hashrefs, the first being the document notation schema, and the
+second being the hierarchical data you wish to validate.
+
 =head1 AUTHOR
 
 Al Newkirk <anewkirk@ana.io>
